@@ -40,12 +40,21 @@
  *	Author: Sasha K (sashak@smlink.com)
  *
  */
-
+ 
+ /*
+  *	20070505: -Added Motorola Venor ID and rename Motorola device 5600
+  * http://linmodems.technion.ac.il/bigarch/archive-seventh/msg00846.html
+  * Marvin Stodolsky (marvin.stodolsky@gmail.com)
+  * Alvaro Aguirre (alvaro.aguirre@gmail.com)
+  */
+ 
 /*****************************************************************************/
 
-#include <linux/config.h>
-#include <linux/module.h>
 #include <linux/version.h>
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
+#include <linux/config.h>
+#endif
+#include <linux/module.h>
 #include <linux/init.h>
 #include <linux/pci.h>
 #include <linux/kernel.h>
@@ -53,27 +62,42 @@
 #include <linux/poll.h>
 #include <linux/fs.h>
 #include <asm/uaccess.h>
-#include <linux/devfs_fs_kernel.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,31)
+#include <linux/sched.h>
+#endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 #define OLD_KERNEL 1
 #endif
 
 #ifdef OLD_KERNEL
+#include <linux/devfs_fs_kernel.h>
 #define iminor(i) MINOR((i)->i_rdev)
 #else
 #include <linux/device.h>
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,13)
+#define class_simple_device_add(class, dev, addr, name, i)
+#define class_simple_device_remove(dev)
+#define class_simple_create(module, name) (struct class_simple *)(name)
+#define class_simple_destroy(class)
+#else
+#define pci_match_id(tbl, dev) pci_match_device(tbl, dev)
+#endif
+
 #include <modem_defs.h>
 
 #define AMRMO_MODULE_NAME "slamr"
-#define AMRMO_MAJOR 212
+#define AMRMO_MAJOR 242
 
 #define AMRMO_DBG(fmt...) if (debug > 0) { printk(KERN_DEBUG AMRMO_MODULE_NAME ": " fmt) ; }
 
 #define MAXNUM 4
 
+#ifndef IRQF_SHARED
+#define IRQF_SHARED SA_SHIRQ
+#endif
 
 /* modem cards ids list */
 
@@ -135,6 +159,15 @@
 #define PCI_DEVICE_ID_SL1900			0x3052
 #define PCI_DEVICE_ID_ND92XPA                   0x8800 /* ND92XPA */
 
+#define PCI_VENDOR_ID_PCTEL                     0x134d
+#define PCI_DEVICE_ID_HSP1688                   0x2189
+
+/* Motorola section */
+#define PCI_VENDOR_ID_MOTOROLA          	0x1057
+
+/* Silicon Instruments */
+#define PCI_VENDOR_ID_SILICON                   0x1543
+
 enum {
 	ALS300_CARD = 1,
 	VIA3058_CARD,
@@ -150,7 +183,8 @@ enum {
 	SL1500_CARD,
 	SL1801_CARD,
 	SL1900_CARD,
-	SL2800_CARD
+	SL2800_CARD,
+	SL1543_CARD
 };
 
 
@@ -208,14 +242,21 @@ static const char *card_names[] = {
 	"SL1500",
 	"ALI545A",
 	"SL1900",
-	"SL2800"
+	"SL2800",
+	"SL1543"
 };
 
 
 static struct pci_device_id amrmo_pci_tbl [] __devinitdata = {
-	{PCI_VENDOR_ID_SMARTLINK_1, PCI_DEVICE_ID_SL1900,    /* 163c:3052 */
+	{PCI_VENDOR_ID_SILICON, PCI_DEVICE_ID_SL1900,	   /* SiliconInstr SL1543:3052 */
+	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, SL1543_CARD},
+	{PCI_VENDOR_ID_MOTOROLA, PCI_DEVICE_ID_SL1900,     /* Motorola 1057:3052 */
 	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, SL1900_CARD},
-	{PCI_VENDOR_ID_SMARTLINK_2, PCI_DEVICE_ID_SL1900,    /* 10a5:3052 */
+	{PCI_VENDOR_ID_SMARTLINK_1, PCI_DEVICE_ID_SL1900,  /* 163c:3052 */
+	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, SL1900_CARD},
+	{PCI_VENDOR_ID_SMARTLINK_2, PCI_DEVICE_ID_SL1900,  /* 10a5:3052 */
+	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, SL1900_CARD},
+	{PCI_VENDOR_ID_PCTEL, PCI_DEVICE_ID_HSP1688,       /* 134d:2189 */
 	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, SL1900_CARD},
 	{PCI_VENDOR_ID_PHILIPS,  PCI_DEVICE_ID_UCB1500,    /* 1131:3400 */
 	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, SL1500_CARD},
@@ -250,6 +291,10 @@ static struct pci_device_id amrmo_pci_tbl [] __devinitdata = {
 	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ICH4_CARD},
        	{PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_ICH5,          /* 8086:24d6 */
 	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ICH4_CARD},
+       	{PCI_VENDOR_ID_INTEL, 0x266d,                      /* ICH6 */
+	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ICH4_CARD},
+       	{PCI_VENDOR_ID_INTEL, 0x27dd,                      /* ICH7 */
+	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, ICH4_CARD},
 	{PCI_VENDOR_ID_VIA, PCI_DEVICE_ID_VIA,             /* 1106:3068 */
 	 PCI_ANY_ID, PCI_ANY_ID, 0, 0, VIA3058_CARD},
 	{PCI_VENDOR_ID_SIS, PCI_DEVICE_ID_SIS630,          /* 1039:7013 */
@@ -350,7 +395,8 @@ static ssize_t amrmo_read(struct file *file, char *buffer, size_t count, loff_t
 	if (count > sizeof(amrmo->ibuf))
 		count = sizeof(amrmo->ibuf);
 	ret = amrmo_card_read(amrmo->card,amrmo->ibuf,count);
-	copy_to_user(buffer,amrmo->ibuf,ret);
+	if(copy_to_user(buffer,amrmo->ibuf,ret))
+		return -EFAULT;
 	return ret;
 }
 
@@ -363,7 +409,8 @@ static ssize_t amrmo_write(struct file *file, const char *buffer, size_t count,
 		return -EFAULT;
 	if (count > sizeof(amrmo->obuf))
 		count = sizeof(amrmo->obuf);
-	copy_from_user(amrmo->obuf,buffer,count);
+	if(copy_from_user(amrmo->obuf,buffer,count))
+		return -EFAULT;
 	ret = amrmo_card_write(amrmo->card,amrmo->obuf,count);
 	return ret;
 }
@@ -390,12 +437,17 @@ static unsigned int amrmo_poll(struct file *file, poll_table *wait)
 
 
 
-static int amrmo_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
+static long amrmo_unlocked_ioctl(struct file *file, unsigned int cmd, unsigned long parg)
 {
 	struct amrmo_struct *amrmo = (struct amrmo_struct *)file->private_data;
         unsigned long flags;
-	int ret;
+	long ret;
 	unsigned stat;
+	unsigned long arg=0;
+
+	cmd -= 100000;
+	if (parg)
+		copy_from_user(&arg, (unsigned long *)parg, sizeof(arg));
 	AMRMO_DBG("amrmo_ioctl: cmd %x, arg %lx...\n",cmd,arg);
         switch (cmd) 
         {
@@ -404,7 +456,7 @@ static int amrmo_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
 		stat = amrmo->status;
 		amrmo->status = 0;
 		spin_unlock_irqrestore(&amrmo->lock,flags);
-                if (put_user(stat, (unsigned *) arg))
+                if (put_user(stat, (unsigned *) parg))
                         return -EFAULT;
 		return 0;
 	case MDMCTL_START:
@@ -462,7 +514,7 @@ static struct file_operations amrmo_fops = {
         .read =    amrmo_read,
         .write =   amrmo_write,
         .poll =    amrmo_poll,
-        .ioctl =   amrmo_ioctl,
+        .unlocked_ioctl =   amrmo_unlocked_ioctl,
         .open =    amrmo_open,
         .release = amrmo_release,
 };
@@ -481,7 +533,11 @@ static void amrmo_pci_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 	amrmo_card_interrupt(amrmo->card);
 }
 #else
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
 static irqreturn_t amrmo_pci_interrupt(int irq, void *dev_id, struct pt_regs *regs)
+#else
+static irqreturn_t amrmo_pci_interrupt(int irq, void *dev_id)
+#endif
 {
         struct amrmo_struct *amrmo = (struct amrmo_struct *)dev_id;
 	amrmo_card_interrupt(amrmo->card);
@@ -565,7 +621,7 @@ static int __init amrmo_pci_probe(struct pci_dev *pci_dev, const struct pci_devi
 		}
 	}
 
-        ret = request_irq(amrmo->irq, &amrmo_pci_interrupt,SA_SHIRQ,
+        ret = request_irq(amrmo->irq, &amrmo_pci_interrupt,IRQF_SHARED,
 			  amrmo->name,amrmo);
 	if(ret) {
 		printk(KERN_ERR "slamr: failed request_irq\n");
@@ -621,7 +677,6 @@ static int __init amrmo_pci_probe(struct pci_dev *pci_dev, const struct pci_devi
 #endif
 #else
 	class_simple_device_add(amrmo_class, MKDEV(AMRMO_MAJOR, i), NULL, "slamr%d", i);
-	devfs_mk_cdev(MKDEV(AMRMO_MAJOR,i), S_IFCHR|S_IRUSR|S_IWUSR, "slamr%d", i);
 #endif
 	return 0;
 
@@ -653,7 +708,6 @@ static void __exit amrmo_pci_remove(struct pci_dev *pci_dev)
 #endif
 #else
 	class_simple_device_remove(MKDEV(AMRMO_MAJOR, amrmo->num));
-	devfs_remove("slamr%d", amrmo->num);
 #endif
 	amrmo_table[amrmo->num] = NULL;
 	amrmo_card_disable(amrmo->card);
@@ -681,7 +735,7 @@ static struct pci_driver amrmo_pci_driver = {
  *  module stuff
  */
 
-MODULE_PARM(debug,"i");
+module_param(debug, int, 1);
 MODULE_PARM_DESC(debug,"debug level: 0-3 (default=0)");
 
 MODULE_AUTHOR("Smart Link Ltd.");
@@ -692,6 +746,7 @@ MODULE_LICENSE("Smart Link Ltd.");
 static int __init amrmo_init(void)
 {
 	struct pci_dev *dev = NULL;
+	int err;
 #ifdef OLD_KERNEL
 	if (!pci_present())
 		return -ENODEV;
@@ -706,9 +761,10 @@ static int __init amrmo_init(void)
 #ifdef OLD_KERNEL
 	pci_for_each_dev(dev) {
 #else
-        while ((dev = pci_find_device(PCI_ANY_ID, PCI_ANY_ID, dev)) != NULL) {
+      /*while ((dev = pci_find_device(PCI_ANY_ID, PCI_ANY_ID, dev)) != NULL) {*/
+        while ((dev = pci_get_device(PCI_ANY_ID, PCI_ANY_ID, dev)) != NULL) {
 #endif
-		if(pci_match_device(amrmo_pci_tbl, dev) &&
+		if(pci_match_id(amrmo_pci_tbl, dev) &&
 		   pci_dev_driver(dev)) {
 #ifdef OLD_KERNEL
 			AMRMO_DBG("device %04x:%04x is used by %s: remove\n",
@@ -717,7 +773,7 @@ static int __init amrmo_init(void)
 			if (dev->driver && dev->driver->remove)
 				dev->driver->remove(dev);
 			dev->driver = NULL;
-#else
+#elif LINUX_VERSION_CODE < KERNEL_VERSION(2,6,10)
 			struct device *reldev = get_device(&dev->dev);
 			AMRMO_DBG("device %04x:%04x is grabbed by driver %s: try to release\n",
 				  dev->vendor,dev->device,
@@ -727,8 +783,15 @@ static int __init amrmo_init(void)
 				device_release_driver(reldev);
 				put_device(reldev);
 			}
+#else
+			printk("slamr: device %04x:%04x is grabbed by another driver\n",
+			       dev->vendor,dev->device);
 #endif
+
 		}
+#ifndef OLD_KERNEL
+		pci_dev_put(dev);
+#endif
 	}
 
 #ifndef OLD_KERNEL
@@ -739,13 +802,16 @@ static int __init amrmo_init(void)
 		return err;
 	}
 #endif
-
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,10)
 	if (!pci_register_driver(&amrmo_pci_driver)) {
+#else
+	if ((err = pci_register_driver(&amrmo_pci_driver)) < 0) {
+#endif
 		pci_unregister_driver(&amrmo_pci_driver);
 #ifndef OLD_KERNEL
 		class_simple_destroy(amrmo_class);
 #endif
-                return -ENODEV;
+                return err;
 	}
 
 	if(register_chrdev(AMRMO_MAJOR, "slamr", &amrmo_fops) < 0) {
@@ -775,5 +841,3 @@ module_exit(amrmo_exit);
 #ifdef OLD_KERNEL
 EXPORT_NO_SYMBOLS;
 #endif
-
-
