@@ -62,6 +62,9 @@
 #define ALSA_PCM_NEW_HW_PARAMS_API 1
 #define ALSA_PCM_NEW_SW_PARAMS_API 1
 #include <alsa/asoundlib.h>
+/* buffer size in periods */
+#define BUFFER_PERIODS		12
+#define SHORT_BUFFER_PERIODS	4
 #endif
 
 #include <modem.h>
@@ -85,13 +88,15 @@ extern int  prop_dp_init(void);
 extern void prop_dp_exit(void);
 extern int datafile_load_info(char *name,struct dsp_info *info);
 extern int datafile_save_info(char *name,struct dsp_info *info);
+extern int modem_ring_detector_start(struct modem *m);
 
 /* global config data */
 extern const char *modem_dev_name;
-extern unsigned need_realtime;
+extern unsigned int ring_detector;
+extern unsigned int need_realtime;
 extern const char *modem_group;
 extern mode_t modem_perm;
-
+extern unsigned int use_short_buffer;
 
 
 struct device_struct {
@@ -100,7 +105,12 @@ struct device_struct {
 #ifdef SUPPORT_ALSA
 	snd_pcm_t *phandle;
 	snd_pcm_t *chandle;
+	snd_mixer_t *mhandle;
+	snd_mixer_elem_t *hook_off_elem;
+	snd_mixer_elem_t *cid_elem;
+	snd_mixer_elem_t *speaker_elem;
 	unsigned int period;
+	unsigned int buf_periods;
 	unsigned int started;
 #endif
 	int delay;
@@ -123,26 +133,86 @@ static char outbuf[4096];
 extern unsigned use_alsa;
 static snd_output_t *dbg_out = NULL;
 
+static int alsa_mixer_setup(struct device_struct *dev, const char *dev_name)
+{
+	char card_name[32];
+	int card_num = 0;
+	char *p;
+	snd_mixer_elem_t *elem;
+	int err;
+
+	if((p = strchr(dev_name, ':')))
+		card_num = strtoul(p+1, NULL, 0);
+	sprintf(card_name, "hw:%d", card_num);
+	
+	err = snd_mixer_open(&dev->mhandle, 0);
+	if(err < 0) {
+		DBG("mixer setup: cannot open: %s\n", snd_strerror(err));
+		return err;
+	}
+	err = snd_mixer_attach(dev->mhandle, card_name);
+	if (err < 0) {
+		ERR("mixer setup: attach %s error: %s\n", card_name, snd_strerror(err));
+		goto error;
+	}
+	err = snd_mixer_selem_register(dev->mhandle, NULL, NULL);
+	if (err <0) {
+		ERR("mixer setup: register %s error: %s\n", card_name, snd_strerror(err));
+		goto error;
+	}
+	err = snd_mixer_load(dev->mhandle);
+	if (err < 0) {
+		ERR("mixer setup: load %s error: %s\n", card_name, snd_strerror(err));
+		goto error;
+	}
+	
+	for (elem = snd_mixer_first_elem(dev->mhandle) ; elem; elem = snd_mixer_elem_next(elem)) {
+		if(strcmp(snd_mixer_selem_get_name(elem),"Off-hook") == 0)
+			dev->hook_off_elem = elem;
+		else if(strcmp(snd_mixer_selem_get_name(elem),"Caller ID") == 0)
+			dev->cid_elem = elem;
+		else if(strcmp(snd_mixer_selem_get_name(elem),"Modem Speaker") == 0)
+			dev->speaker_elem = elem;
+	}
+
+	if(dev->hook_off_elem)
+		return 0;
+
+error:
+	snd_mixer_close(dev->mhandle);
+	dev->mhandle = NULL;
+	if (!err) {
+		ERR("mixer setup: Off-hook switch not found for card %s\n", card_name);
+		err = -ENODEV;
+	}
+	return err;
+}
+
 static int alsa_device_setup(struct device_struct *dev, const char *dev_name)
 {
 	struct pollfd pfd;
 	int ret;
 	memset(dev,0,sizeof(*dev));
+
+	ret = alsa_mixer_setup(dev, dev_name);
+	if(ret < 0)
+		DBG("alsa setup: cannot setup mixer: %s\n", snd_strerror(ret));
+
 	ret = snd_pcm_open(&dev->phandle, dev_name, SND_PCM_STREAM_PLAYBACK, SND_PCM_NONBLOCK);
 	if(ret < 0) {
-		ERR("alsa setup: cannot open playback device '%s': %s",
+		ERR("alsa setup: cannot open playback device '%s': %s\n",
 		    dev_name, snd_strerror(ret));
 		return -1;
 	}
 	ret = snd_pcm_open(&dev->chandle, dev_name, SND_PCM_STREAM_CAPTURE, SND_PCM_NONBLOCK);
 	if(ret < 0) {
-		ERR("alsa setup: cannot open playback device '%s': %s",
+		ERR("alsa setup: cannot open playback device '%s': %s\n",
 		    dev_name, snd_strerror(ret));
 		return -1;
 	}
 	ret = snd_pcm_poll_descriptors(dev->chandle, &pfd, 1);
 	if(ret <= 0) {
-		ERR("alsa setup: cannot get poll descriptors of '%s': %s",
+		ERR("alsa setup: cannot get poll descriptors of '%s': %s\n",
 		    dev_name, snd_strerror(ret));
 		return -1;
 	}
@@ -159,6 +229,15 @@ static int alsa_device_release(struct device_struct *dev)
 {
 	snd_pcm_close (dev->phandle);
 	snd_pcm_close (dev->chandle);
+	if (dev->mhandle) {
+		if (dev->hook_off_elem)
+			snd_mixer_selem_set_playback_switch_all(dev->hook_off_elem, 0);
+		if (dev->cid_elem)
+			snd_mixer_selem_set_playback_switch_all(dev->cid_elem, 0);
+		if (dev->speaker_elem)
+			snd_mixer_selem_set_playback_switch_all(dev->speaker_elem, 0);
+		snd_mixer_close(dev->mhandle);
+	}
 	return 0;
 }
 
@@ -200,7 +279,7 @@ static int alsa_device_read(struct device_struct *dev, char *buf, int count)
 			break;
 		}
 	} while (ret == -EAGAIN);
-#if 1
+#if 0
 	if(ret != dev->period)
 		DBG("alsa_device_read (%d): %d ...\n",count,ret);
 #endif
@@ -209,7 +288,6 @@ static int alsa_device_read(struct device_struct *dev, char *buf, int count)
 
 static int alsa_device_write(struct device_struct *dev, const char *buf, int count)
 {
-	int asked = count;
 	int written = 0;
 	if(!dev->started)
 		return 0;
@@ -228,7 +306,7 @@ static int alsa_device_write(struct device_struct *dev, const char *buf, int cou
 		buf += ret;
 		written += ret;
 	}
-#if 1
+#if 0
 	if(written != dev->period)
 		DBG("alsa_device_write (%d): %d...\n",asked,written);
 #endif
@@ -301,12 +379,12 @@ static int setup_stream(snd_pcm_t *handle, struct modem *m, const char *stream_n
 		ERR("cannot set periods for %s: %s\n", stream_name, snd_strerror(err));
 		return err;
 	}
-	if ( rsize != size ) {
+	if ( rsize < size ) {
 		ERR("period size %ld is not supported by %s (%ld).\n",
 		    size, stream_name, rsize);
 		return -1;		
 	}
-	rsize = size = rsize * 32;
+	rsize = size = use_short_buffer ? rsize * dev->buf_periods : rsize * 32;
 	err = snd_pcm_hw_params_set_buffer_size_near(handle, hw_params, &rsize);
 	if (err < 0) {
 		ERR("cannot set buffer for %s: %s\n", stream_name, snd_strerror(err));
@@ -371,6 +449,7 @@ static int alsa_start (struct modem *m)
 	int err, len;
 	DBG("alsa_start...\n");
 	dev->period = m->frag;
+	dev->buf_periods = use_short_buffer ? SHORT_BUFFER_PERIODS : BUFFER_PERIODS;
 	err = setup_stream(dev->phandle, m, "playback");
 	if(err < 0)
 		return err;
@@ -378,7 +457,7 @@ static int alsa_start (struct modem *m)
 	if(err < 0)
 		return err;
 	dev->delay = 0;
-	len = 384;           /* <-- FIXME */
+	len = use_short_buffer ? dev->period * dev->buf_periods : 384;
 	DBG("startup write: %d...\n",len);
 	err = snd_pcm_format_set_silence(SND_PCM_FORMAT_S16_LE, outbuf, len);
 	if(err < 0) {
@@ -431,7 +510,14 @@ static int alsa_ioctl(struct modem *m, unsigned int cmd, unsigned long arg)
         case MDMCTL_CAPABILITIES:
                 return -EINVAL;
         case MDMCTL_HOOKSTATE:
-                return 0;
+		return (dev->hook_off_elem) ?
+			snd_mixer_selem_set_playback_switch_all(
+				dev->hook_off_elem,
+				(arg == MODEM_HOOK_OFF) ) : 0 ;
+	case MDMCTL_SPEAKERVOL:
+		return (dev->speaker_elem) ?
+			snd_mixer_selem_set_playback_volume_all(
+					dev->speaker_elem, arg) : 0 ;
         case MDMCTL_CODECTYPE:
                 return CODEC_SILABS;
         case MDMCTL_IODELAY:
@@ -466,14 +552,14 @@ static int modemap_start (struct modem *m)
 	int ret;
 	DBG("modemap_start...\n");
 	dev->delay = 0;
-        ret = ioctl(dev->fd,MDMCTL_START,0);
+        ret = ioctl(dev->fd,100000+MDMCTL_START,0);
 	if (ret < 0)
 		return ret;
 	ret = 192*2;
 	memset(outbuf, 0 , ret);
 	ret = write(dev->fd, outbuf, ret);
 	if (ret < 0) {
-		ioctl(dev->fd,MDMCTL_STOP,0);
+		ioctl(dev->fd,100000+MDMCTL_STOP,0);
 		return ret;
 	}
 	dev->delay = ret/2;
@@ -484,7 +570,7 @@ static int modemap_stop (struct modem *m)
 {
 	struct device_struct *dev = m->dev_data;
 	DBG("modemap_stop...\n");
-        return ioctl(dev->fd,MDMCTL_STOP,0);
+        return ioctl(dev->fd,100000+MDMCTL_STOP,0);
 }
 
 static int modemap_ioctl(struct modem *m, unsigned int cmd, unsigned long arg)
@@ -494,7 +580,7 @@ static int modemap_ioctl(struct modem *m, unsigned int cmd, unsigned long arg)
 	DBG("modemap_ioctl: cmd %x, arg %lx...\n",cmd,arg);
 	if (cmd == MDMCTL_SETFRAG)
 		arg <<= MFMT_SHIFT(m->format);
-	ret = ioctl(dev->fd,cmd,arg);
+	ret = ioctl(dev->fd,cmd+100000,&arg);
 	if (cmd == MDMCTL_IODELAY && ret > 0) {
 		ret >>= MFMT_SHIFT(m->format);
 		ret += dev->delay;
@@ -678,12 +764,18 @@ static int modem_run(struct modem *m, struct device_struct *dev)
 		if(m->event)
 			modem_event(m);
 
+#ifdef MODEM_CONFIG_RING_DETECTOR
+		if(ring_detector && !m->started)
+			modem_ring_detector_start(m);
+#endif
+
                 tmo.tv_sec = 1;
                 tmo.tv_usec= 0;
                 FD_ZERO(&rset);
 		FD_ZERO(&eset);
 		if(m->started)
 			FD_SET(dev->fd,&rset);
+
 		FD_SET(dev->fd,&eset);
 		max_fd = dev->fd;
 		
@@ -718,7 +810,7 @@ static int modem_run(struct modem *m, struct device_struct *dev)
 				continue;
 			}
 #endif
-			ret = ioctl(dev->fd,MDMCTL_GETSTAT,&stat);
+			ret = ioctl(dev->fd,100000+MDMCTL_GETSTAT,&stat);
 			if(ret < 0) {
 				ERR("dev ioctl: %s\n",strerror(errno));
 				return -1;
